; File:		macros.asm
; Device:	AVR
; Version:	2024-05-06
; Author:	Johannes Fechner
;			https://4n7.de/
;			https://github.com/jofe95

#define ROUND(X) (int(1.0*(X)+0.5))
#define FLOOR(X) (int(X))
#define CEIL(X) (frac(X) > 0 ? int(X)+1 : int(X))

.macro xout ; eXtended "out"
	.if @0 > 0x3F
		sts @0, @1
	.else
		out @0, @1
	.endif
.endmacro

.macro xin ; eXtended "in"
	.if @1 > 0x3F
		lds @0, @1
	.else
		in  @0, @1
	.endif
.endmacro

.macro ldiz ; Load immediate into Z double register.
	ldi		ZH, high(@0)
	ldi		ZL, low(@0)
.endmacro

.macro addz ; Add register to Z double register; register 'zero' must be cleared.
	add 	ZL, @0
	adc		ZH, zero
.endmacro

.macro addi16 ; RdH, RdL, k
	subi    @1, low(-@2)
	sbci    @0, high(-@2)
.endmacro

.macro	wait_ns_		; ns, -Takte					; nach 'Klaus2m5'
.set cycle_time_ns = (1_000_000_000 / F_CPU)
.set cycles = ((@0 + cycle_time_ns - 1) / cycle_time_ns - @1)
.if (cycles > (255 * 3 + 2))
.error "wait_ns_ overflow"
.else
.if (cycles > 6)
.set  loop_cycles = (cycles / 3)      
	ldi		tmp0,loop_cycles
	 dec	tmp0
	brne	pc-1
.set  cycles = (cycles - (loop_cycles * 3))
.endif
.if (cycles > 0)
.if (cycles & 4)
	rjmp	pc+1
	rjmp	pc+1
.endif
.if (cycles & 2)
	rjmp	pc+1
.endif
.if (cycles & 1)
	nop
.endif
.endif
.endif
.endmacro
