; == UART routines ==
; === Routine: Send character via UART ===
; Parameter: tmp0.
uart_putChar:
	push	tmp0
; Make sure the transmit data registers are empty:
uart_sendByte_wait:
	xin		tmp0, USART0_STATUS
	sbrs	tmp0, USART_DREIF_bp
	rjmp	uart_sendByte_wait
	pop		tmp0
	xout	USART0_TXDATAL, tmp0
	ret

; === Routine: Send string via UART ===
; Parameter: Z pointer, start address of the null-terminated string.
; String length is limited to 255 characters.
uart_putSz:
	push	tmp0
	push	tmp1
	clr		tmp1
uart_putSz_loop:
	lpm		tmp0, Z+
	tst		tmp0
	breq	uart_putSz_end
	rcall	uart_putChar
	inc		tmp1
	cpi		tmp1, 0xFF
	breq	uart_putSz_end
	rjmp	uart_putSz_loop
uart_putSz_end:
	pop		tmp1
	pop		tmp0
	ret

; === Routine: Send byte in hexadecimal via UART ===
; Send 8-bit unsigned number as two hex digits.
; Parameter: tmp0.
uart_putHexByte:
	push	tmp0
	swap	tmp0
	andi	tmp0, 0x0F
	rcall	uart_putHexDigit
	pop		tmp0
	push	tmp0
	andi	tmp0, 0x0F
	rcall	uart_putHexDigit
	pop		tmp0
	ret
uart_putHexDigit:
	cpi		tmp0, 10
	brlt	uart_putHexDigit_1
	subi	tmp0, -('A'-'9'-1)
uart_putHexDigit_1:
	subi	tmp0, -'0'
	rcall	uart_putChar
	ret

; === Routine: Wait for reception of character from UART ===
; Return: tmp0, the received character.
uart_getChar:
	xin		tmp0, USART0_STATUS
	sbrs	tmp0, USART_RXCIF_bp
	rjmp	uart_getChar
	xin		tmp0, USART0_RXDATAL
	ret
	
; === Routine: Convert ASCII hex digit character to its value ===
; Parameter:	tmp0, the hex digit (ASCII) to be converted.
; Return:		tmp1, $FF if given character is no hex digit.
hexChar2num:
	mov		tmp1, tmp0
; Convert small letters to capitals:
	sbrc	tmp1, 6
	andi	tmp1, ~(1<<5)
	cpi		tmp1, '0'
	brlo	hexChar2num_err
	cpi		tmp1, '9'+1
	brlo	hexChar2num_09
	cpi		tmp1, 'A'
	brlo	hexChar2num_err
	cpi		tmp1, 'F'+1
	brlo	hexChar2num_AF
hexChar2num_err:
	ldi		tmp1, 0xFF
	rjmp	hexChar2num_end
hexChar2num_09:
	subi	tmp1, '0'
	rjmp	hexChar2num_end
hexChar2num_AF:
	subi	tmp1, 'A'-10
hexChar2num_end:
	ret

; == Routine: Read byte as two hex chars from UART ==
; Return:		tmp0.
uart_getHexByte:
	push	tmp1
	push	tmp2
uart_getHexByte_get1stChar:
	rcall	uart_getChar
	rcall	hexChar2num
	cpi		tmp1, 0xFF
	breq	uart_getHexByte_get1stChar
	rcall	uart_putChar				; Echo the first entered character.
	mov		tmp2, tmp1
uart_getHexByte_get2ndChar:
	rcall	uart_getChar
	rcall	hexChar2num
	cpi		tmp1, 0xFF
	breq	uart_getHexByte_get2ndChar
	rcall	uart_putChar				; Echo the second entered character.
	swap	tmp2
	or		tmp1, tmp2
	mov		tmp0, tmp1
	pop		tmp2
	pop		tmp1
	ret
